// robowaifu access control
// ========================
// -The software testbed for devising access control systems for robowaifus

// -Filename: Human.cpp
// -The definitions of the Human class

#include <algorithm>
#include <iostream>  // TEMP
#include <iterator>

#include "Human.hpp"

using std::cbegin;
using std::cend;
using std::find;

//------------------------------------------------------------------------------
Human::Human(std::string const& name, Groups const& groups)
    : name{name}, groups{groups} {
  //
  // TODO DESIGN: later, this will be part of a secured system start-up process,
  // and will not be within a general Human ctor check/assignment
  //
  // TODO SEC: robustify this approach. spoofing seems a possibility as-is.
  //   -also, ensure there _IS_ a 'The_Master' for robowaifu after init-setup.
  //   not having one is an error state (and a huge security risk potential).
  //
  // validate the one (and _only_ one) Master for robowaifu
  //
  if (! rw::The_Master) {  // check that no 'The_Master' assignment already made
    //
    // TODO DESIGN: move this into a general utility container search function
    //   -return an iterator/bool pair
    //   -discriminate responses between sensitive requests (such as this exact
    //   'Relation::master' one), and more commonplace searches
    //
    // discover if this Human is 'The_Master'
    if (const auto it = find(cbegin(groups), cend(groups), Relation::master);
        it != cend(groups)) {
      //
      // std::cerr << "Found 'The_Master'\n";  // TEST TEMP

      // -found him, so assign this object's pointer to the static var
      rw::The_Master = std::make_unique<Human>(*this);
    }
  }

  // validate this Human is included in at least one Group
  if (! groups.size()) {
    // TODO DESIGN: for now, simply display an error. later, throw an exception?
    const auto err_msg{"\n'" + name +
                       "' ERR: a Human must be part of at least one Group\n\n"};
    std::cerr << err_msg;
  }
}

//------------------------------------------------------------------------------
auto Human::human() const noexcept -> decltype(std::string{}) {
  return name;
}

//------------------------------------------------------------------------------
auto Human::relations() const noexcept -> decltype(Groups{}) {
  return groups;
}

// Copyright (2021)
// License MIT (Expat) https://opensource.org/licenses/MIT
