// robowaifu access control
// ========================
// -The software testbed for devising access control systems for robowaifus

// -Filename: Sequence.hpp
// -The declarations interface of the Sequence class

// TODO DOC: Add exposition for Anon as a learner, to understand why & how
// Sequence objects are used in this system.

#pragma once

#include <string>

#include "Human.hpp"
#include "utility_types.hpp"

using std::string;

/** Represents a sequence of automation instructions within robowaifu's
 * model-of-the-world, with the associated Human ownership & authorized
 * container of Relation's (Permit) embedded within */
class Sequence {
 public:
  Sequence(){};
  /** Basic Sequence ctor
   * @param instr_seq [IN] this Sequence's data load
   * @param owner [IN] the Human in control of this Sequence
   * @param permits [IN] the container of Relation's this Sequence is authorized
   * for
   */
  Sequence(std::string const& instr_seq,
           Human const&       owner,
           Permit const&      permits);

  /** @return the Sequence's instruction sequence */
  auto info() const noexcept -> decltype(std::string{});

  /** @return the Sequence's container of Relation's it is authorized for */
  auto allows() const noexcept -> decltype(Permit{});

 private:
  string instr_seq;
  Human  owner;
  Permit permits;
};

// Copyright (2021)
// License MIT (Expat) https://opensource.org/licenses/MIT
