// robowaifu access control
// ========================
// -The software testbed for devising access control systems for robowaifus

// -Filename: rw_cli_args.hpp
// -Command-line parsing & help support functions.

#pragma once

#include <cstdlib>

#include "rw_gen_utils.hpp"
#include "rw_user_args.hpp"

namespace rw {

// This is the single, and overall CLI App object for parsing program arguments
CLI::App cli_app{"robowaifu access control, access control for robowaifus\n"};

// Simple flag to track if the help flag was passed in
bool have_help{false};

/**-----------------------------------------------------------------------------
 @brief    Configure for, and parse, any command-line arguments.

 @param    argc [IN] The CLI arguments count.
 @param    argv [IN] The CLI arguments C-string array.
 @returns  Success code 0, or a parse error code.
*/
int handle_args(int argc, char** argv) {
  //

  // https://github.com/CLIUtils/CLI11/blob/master/examples/modhelp.cpp
  // Disable the default help handler...
  cli_app.set_help_flag();
  // and add a custom flag for activating help
  auto help = cli_app.add_flag("-h, --help", "Display this help and quit");

  try {
    cli_app.parse(argc, argv);

    if (*help) {                 // Special block just for the custom help flag
      have_help = true;          // used by parse_args() below
      throw CLI::CallForHelp();  // caught immediately below, returns a '0' exit
    }
  } catch (const CLI::ParseError& e) {
    return cli_app.exit(e);
  }

  return 0;
}

/**-----------------------------------------------------------------------------
 @brief    Wrapper to unify correct program exit on either arg errors or the
 help flag.
*/
void parse_args(int argc, char** argv) {
  if (auto bad_arg = handle_args(argc, argv)) {  // non-zero ret == error state
    std::exit(bad_arg);
  } else if (have_help) {  // simple hack to prevent program execution on --help
    std::exit(0);
  }
}

/**-----------------------------------------------------------------------------
 @brief    Display the help before exit.
*/
void help_exit() {
  //
  cli_app.exit(CLI::CallForHelp());
  std::exit(0);
}

}  // namespace rw

// Copyright (2021)
// License MIT (Expat) https://opensource.org/licenses/MIT
